/*
 Acceler Box
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 18/05/2013
 Last Modified: 22/05/2013
 
 A small box with LEDs that blink when the box is moved depending on how much g was sensed
 
 */

#define F_CPU 1000000 // 1 MHz

#define x_axis 3 // ADC3 (PB3)
#define y_axis 2 // ADC2 (PB4)
#define z_axis 1 // ADC1 (PB2)
#define Acc_sleep PB1
#define led PB0

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "setup.c"

// ATtiny25/45/85 Pin map
//
//              +-\/-+
//  Reset PB5  1|o   |8  VCC
//      X PB3  2|    |7  PB2 Z
//      Y PB4  3|    |6  PB1 Sleep Pin
//        GND  4|    |5  PB0 LED
//              +----+

int main(void) {
	setup();
	
	while(1) {
		// Read the initial X, Y, Z axis
		int x_result = adcRead(x_axis);
		int y_result = adcRead(y_axis);
		int z_result = adcRead(z_axis);
		
		// Re-read X, Y, Z axis for 500ms
		int highest_value = 0;
		for (int x = 0; x < 1000; x++) {
			int x_change = adcRead(x_axis) - x_result;
			int y_change = adcRead(y_axis) - y_result;
			int z_change = adcRead(z_axis) - z_result;
			
			// Convert negative to positive value
			if (x_change < 0) {
				x_change = x_change - (x_change * 2);
			}
			if (y_change < 0) {
				y_change = y_change - (y_change * 2);
			}
			if (z_change < 0) {
				z_change = z_change - (z_change * 2);
			}
			
			// If there was a 54 result difference (0.2g or more), find out the which axis had the most change
			// 0.1g is 80mV assuming 800mV/g typical. With a 3V battery, 0.16V / 0.0029296875 = 54.64
			if (x_change > 54 || y_change > 54 || z_change > 54) {
				if (x_change > highest_value) {
					highest_value = x_change;
				}
				if (y_change > highest_value) {
					highest_value = y_change;
				}
				if (z_change > highest_value) {
					highest_value = z_change;
				}
			}
		}
		
		// For every 0.1g change blink the LEDs
		for (int x = 0; x < (highest_value / 27); x++) {
			PORTB |= (1<<led);
			watchdogSleep(T128MS);
			PORTB &= ~(1<<led);
			watchdogSleep(T128MS);
		}
		
		// Sleep for a bit
		watchdogSleep(T128MS);
	}
}

// Watchdog timer
ISR(WDT_vect) { }

// ADC interrupt
ISR(ADC_vect) { }
