/*
 Acceler Ball
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 18/05/2013
 Last Modified: 22/05/2013
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(int ii) {
	byte bb;
	if (ii > 9 ) ii=9;
	bb=ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCR = bb;
	WDTCR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCR |= (1<<WDCE) | (1<<WDE);
	WDTCR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdogSleep(int timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turnOffwatchdog();
}


// Read from ADC
int adcRead(uint8_t adc) {
	uint8_t low, high;
	
	// set the analog reference and channel
	ADMUX = (adc & 0x3f);
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// Read results
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}


// Setup the ATtiny
void setup(void) {
	
	// Set Outputs
	DDRB |= (1<<Acc_sleep);
	DDRB |= (1<<led);
	
	// Wake up the accelerometer
	PORTB |= (1<<Acc_sleep);
	
	// Set ADC prescale factor to 8
	// 1 MHz / 8 = 125 KHz, inside the desired 50-200 KHz range.
	sbi(ADCSRA, ADPS1);
	sbi(ADCSRA, ADPS0);
	
	// Enable ADC conversions
	sbi(ADCSRA, ADEN);
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
	
	sei(); // Turn on interrupts
}